/*
SPDX-FileCopyrightText: Metadata Cleaner contributors
SPDX-License-Identifier: GPL-3.0-or-later
*/

.files {
	background: none;
}

listview.files>row {
	padding: 0;
	border-bottom: 1px solid var(--border-color);
}

listview.files .file {
	border-radius: unset;
	font-weight: unset;
}

listview.metadata {
	background: none;
}

listview.metadata > row:first-child {
  margin-top: 6px;
}

listview.metadata > row {
  margin-top: 16px;
}

listview.metadata > row:last-child {
  margin-bottom: 6px;
}

.badge {
	border-radius: 11px;
	background-color: rgba(0, 0, 0, 0.5);
	color: #fff;
	font-weight: bold;
	font-size: 12px;
}

.badge.flat {
	background: none;
	color: var(--window-fg-color);
}

.badge label {
	margin: 0 6px;
}

.badge.metadata {
	background-color: var(--accent-bg-color);
	color: var(--accent-fg-color);
}

.badge.warning {
	background-color: var(--warning-bg-color);
	color: var(--warning-fg-color);
}

.badge.error {
	background-color: var(--error-bg-color);
	color: var(--error-fg-color);
}

.badge.success {
	background-color: var(--success-bg-color);
	color: var(--success-fg-color);
}

.badge label {
	padding: 0 0.4em;
}

.mime-icon {
	filter: drop-shadow(0px 0px 1px rgba(0, 0, 0, 0.4));
}

.dropoverlay,
.dropoverlay > overlay > :not(revealer) {
	box-shadow: none;
}

.dropoverlay .status {
	background-color: color-mix(in srgb, var(--accent-bg-color) 80%, transparent);
	color: var(--accent-fg-color);
}
